<template>
  <el-card class="archive-management-card">
    <div class="left">
      <div class="left-content">
        <div class="tree-header">
          <h3>文档管理</h3>
          <el-button icon="Plus" size="small" type="primary" @click="append('')"
          >新增
          </el-button
          >
        </div>

        <!-- 搜索框 -->
        <div class="search-box">
          <el-input
              v-model="filterText"
              clearable
              placeholder="输入关键字进行搜索"
              size="small"
              @input="handleFilter"
          >
            <template #prefix>
              <el-icon>
                <Search/>
              </el-icon>
            </template>
          </el-input>
        </div>

        <div class="tree-container">
          <el-tree
              ref="treeRef"
              :data="treeData"
              :default-expand-all="false"
              :expand-on-click-node="false"
              :filter-node-method="filterNode"
              :props="props"
              class="custom-tree"
              node-key="id"
              @node-click="handleNodeClick"
          >
            <template #default="{ node, data }">
              <div class="tree-node-content" @dblclick="headerDbClick(node,data)">
                <div class="node-icon">
                  <el-icon
                      v-if="!node.isLeaf"
                      :class="{ expanded: node.expanded }"
                  >
                    <Folder/>
                  </el-icon>
                  <el-icon v-else>
                    <Document/>
                  </el-icon>
                </div>

                <div class="node-label">
                  <span v-if="!data.isEdit" class="label-text">{{
                      node.label
                    }}</span>
                  <el-input
                      v-else
                      :ref="(el) => setInputRef(el, data)"
                      v-model="newName"
                      autofocus
                      class="tree-input"
                      placeholder="请输入节点名称"
                      size="small"
                      @blur="(event) => handleInputBlur(event, data, node)"
                      @keyup.enter="
                      (event) => handleInputBlur(event, data, node)
                    "
                  />
                </div>
                <div v-show="!data.isEdit" class="node-actions">
                  <el-button
                      icon="Plus"
                      link
                      size="small"
                      title="新增子节点"
                      @click.stop="append(data)"
                  ></el-button>
                  <el-button
                      icon="Delete"
                      link
                      size="small"
                      title="删除"
                      @click.stop="remove(node, data)"
                  ></el-button>
                </div>
              </div>
            </template>
          </el-tree>
        </div>
      </div>
    </div>
    <div class="right">
      <el-row :gutter="24">
        <el-col :offset="20" :span="2"
        >
          <el-button :icon="Delete" type="danger" @click="delHandler">删除</el-button>
        </el-col
        >
        <el-col :span="2"
        >
          <el-button
              :disabled="!tableSwitch"
              :icon="Plus"
              type="primary"
              @click="add"
          >新增
          </el-button
          >
        </el-col
        >
      </el-row>
      <ETable
          :border="true"
          :columns="columns"
          :loading="loading"
          :maxHeight="1200"
          :show-selection="true"
          :table-data="tableData"
          @edit="handleEdit"
          @selection-change="handleSelectionChange"
      >
      </ETable>
      <Pagination
          :layout="'total, prev, pager, next, jumper'"
          :limit="queryParams.pageSize"
          :page="queryParams.current"
          :show-total="true"
          :total="total"
          @pagination="handlePageChange"
      ></Pagination>
    </div>
    <archiveDialog
        ref="archiveDialogs"
        v-model:centerDialogVisible="dialogVisible"
        :row="row"
        @centerDialogVisible="centerDialogVisible"
        @submitForm="submitForm"

    >
    </archiveDialog>
  </el-card>
</template>
<script setup>
import {nextTick, onMounted, reactive, ref} from "vue";
import ETable from "@/components/Table/ETable.vue";
import {ElButton, ElIcon, ElInput, ElMessage, ElMessageBox} from "element-plus";
import archiveDialog from "./mould/archiveDialog.vue";
import Pagination from "@/components/Pagination/index.vue";
import {Delete, Document, Folder, Plus, Search,} from "@element-plus/icons-vue";
import {addOrEditTree, delArchive, delTree, getArchiveList, getTree,} from "@/api/archiveManagement";

const dialogVisible = ref(false); // 控制归档对话框显示
const loading = ref(false);
const tableData = ref([]);
const treeData = ref([]);
const newName = ref("");
const inputRefs = ref(new Map()); // 存储输入框引用
const filterText = ref(""); // 搜索关键字
const treeRef = ref(); // 树组件引用
const total = ref(0); // 总记录数
const columns = [
  {prop: "name", label: "名称", minWidth: 180},
  {prop: "type", label: "类型", minWidth: 120},
  {prop: "status", label: "状态", minWidth: 100},
];
const selectedRows = reactive([]); // 存储选中行数据
const handleSelectionChange = (selection) => {
  selectedRows.splice(0, selectedRows.length, ...selection);
};
const queryParams = reactive({
  searchAll: "",
  current: 1,
  pageSize: 10, // 固定每页10条
  treeId: null, // 当前树节点ID
});
// 搜索过滤功能
const handleFilter = () => {
  treeRef.value?.filter(filterText.value);
};
const row = ref({}); // 当前选中行数据
const filterNode = (value, data) => {
  if (!value) return true;
  return data.name?.toLowerCase().includes(value.toLowerCase());
};
const submitForm = async (res) => {
  try {
    if (res && res.code === 200) {
      ElMessage.success("操作成功");
      dialogVisible.value = false;
      // 刷新列表数据
      await getArchiveListData();
    } else {
      ElMessage.error("操作失败: " + (res?.message || res?.msg || "未知错误"));
    }
  } catch (error) {
    console.error("提交表单错误:", error);
    ElMessage.error("操作失败，请稍后重试");
  }
};

const centerDialogVisible = (val) => {
  dialogVisible.value = val;
};
const tableSwitch = ref(false);
// 处理节点点击
const handleNodeClick = (data) => {
  rowClickData.value = data; // 存储当前点击的节点数据
  tableSwitch.value = true;
  // 切换节点时重置到第一页
  queryParams.current = 1;
  queryParams.treeId = data.id;
  getArchiveListData();
};
const rowClickData = ref({}); // 存储当前点击的节点数据
const archiveDialogs = ref(null); // 表格组件引用
// 新增归档
const add = () => {
  try {
    row.value = {}; // 清空行数据，确保是新增模式
    newName.value = ""; // 清空输入框
    dialogVisible.value = true;
    
    // 确保组件引用存在后再调用方法
    nextTick(() => {
      if (archiveDialogs.value && typeof archiveDialogs.value.initForm === 'function') {
        archiveDialogs.value.initForm(rowClickData.value); // 重置表单
      }
    });
  } catch (error) {
    console.error("新增归档错误:", error);
    ElMessage.error("打开新增界面失败");
  }
};
// 处理分页变化
const handlePageChange = (pagination) => {
  try {
    const { page, limit } = pagination;
    queryParams.current = page;
    if (limit) {
      queryParams.pageSize = limit;
    }
    getArchiveListData();
  } catch (error) {
    console.error("分页处理错误:", error);
    ElMessage.error("分页操作失败");
  }
};

const getArchiveListData = async () => {
  try {
    loading.value = true;
    let res = await getArchiveList({
      treeId: queryParams.treeId,
      current: queryParams.current,
      size: queryParams.pageSize,
    });

    if (res.code !== 200) {
      ElMessage.error("获取数据失败: " + res.message);
      tableData.value = [];
      total.value = 0;
      return;
    }
    tableData.value = res.data?.records || res.data || [];
    total.value = res.data?.total || 0;
    // 确保分页参数正确更新
    if (res.data?.current) {
      queryParams.current = res.data.current;
    }
    // pageSize 固定为20，不从后端获取

  } catch (error) {
    ElMessage.error("获取数据失败");
    tableData.value = [];
    total.value = 0;
  } finally {
    loading.value = false;
  }
};
// 删除选中的归档记录
const delHandler = async () => {
  if (selectedRows.length === 0) {
    ElMessage.warning("请选择要删除的数据");
    return;
  }

  try {
    await ElMessageBox.confirm(
      `确定要删除选中的 ${selectedRows.length} 条记录吗？`,
      '删除确认',
      {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning',
      }
    );

    const ids = selectedRows.map((row) => row.id);
    const { code, msg } = await delArchive(ids);
    
    if (code !== 200) {
      ElMessage.error("删除失败: " + msg);
      return;
    }

    ElMessage.success("删除成功");
    // 删除成功后重新获取数据
    await getArchiveListData();
    // 清空选中状态
    selectedRows.splice(0, selectedRows.length);
    
  } catch (error) {
    if (error !== 'cancel') {
      console.error("删除归档失败:", error);
      ElMessage.error("删除操作失败，请稍后重试");
    }
  }
};
// 双击编辑节点
const headerDbClick = (node, data) => {
  data.isEdit = true;
  newName.value = data.name;
  nextTick(() => {
    const inputEl = inputRefs.value.get(data.id || data);
    if (inputEl) {
      inputEl.focus();
      inputEl.select();
    }
  });
};

// 设置输入框引用的方法
const setInputRef = (el, data) => {
  if (el) {
    inputRefs.value.set(data.id || data, el);
    if (data.isEdit) {
      nextTick(() => {
        // el.focus();
        // el.select();
      });
    }
  }
};

// 处理输入框失焦
const handleInputBlur = async (event, comeTreeData, node) => {
  try {
    if (!comeTreeData.isEdit) return; // 如果不是编辑状态，直接返回
    
    if (event.relatedTarget && event.relatedTarget.tagName === "BUTTON") {
      return;
    }
    
    comeTreeData.isEdit = false;
    const newValue = newName.value.trim();
    
    if (comeTreeData.name === newValue) {
      return;
    }
    
    if (newValue === "") {
      newName.value = comeTreeData.name || "新节点";
      ElMessage.warning("节点名称不能为空");
      return;
    }
    
    // 获取父节点的id - 通过 node 参数更准确地获取
    let parentId = null;
    if (node && node.parent && node.parent.data) {
      parentId = node.parent.data.id;
    }
    
    const result = await addOrEditTree({
      name: newValue,
      parentId: parentId || null, // 如果没有父节点，则为 null
      id: comeTreeData.id || null,
    });
      if (result.code === 200) {
      comeTreeData.name = newValue;
      if (!comeTreeData.id && result.data) {
        comeTreeData.id = result.data.id || result.data;
      }
      ElMessage.success("保存成功");
      
      // 刷新树数据并展开当前节点
      const currentNodeId = comeTreeData.id;
      await getList();
      
      // 等待DOM更新后展开节点
      nextTick(() => {
        if (currentNodeId && treeRef.value) {
          const targetNode = treeRef.value.getNode(currentNodeId);
          if (targetNode) {
            // 展开当前节点
            targetNode.expanded = true;
            // 如果有父节点，也展开父节点路径
            expandParentNodes(targetNode);
          }
        }
      });
    } else {
      comeTreeData.name = comeTreeData.name || "新节点";
      ElMessage.error("保存失败: " + (result.msg || "未知错误"));
    }
    
  } catch (error) {
    console.error("保存节点失败:", error);
    comeTreeData.name = comeTreeData.name || "新节点";
    ElMessage.error("保存失败，请稍后重试");
  }
};

onMounted(async () => {
  await getList();
});
const props = {
  label: "name",
  children: "children", // 改为 children 以匹配标准结构
  isLeaf: "leaf",
};

// 展开父节点路径
const expandParentNodes = (node) => {
  if (node && node.parent && node.parent.data) {
    // 递归展开所有父节点
    node.parent.expanded = true;
    expandParentNodes(node.parent);
  }
};

// 删除树节点
const remove = async (node, data) => {
  if (!data || !data.id) {
    ElMessage.warning("无法删除此节点");
    return;
  }

  try {
    await ElMessageBox.confirm(
      `确定要删除节点 "${data.name}" 吗？`,
      '删除确认',
      {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning',
      }
    );

    const { code, msg } = await delTree([data.id]);
    
    if (code !== 200) {
      ElMessage.error("删除失败: " + msg);
      return;
    }

    ElMessage.success("删除成功");
    await getList();
    
  } catch (error) {
    if (error !== 'cancel') {
      console.error("删除节点失败:", error);
      ElMessage.error("删除失败，请稍后重试");
    }
  }
};

const append = async (data) => {
  if (data === "") {
    // 新增根节点
    const newNode = {
      name: "新节点",
      isEdit: true,
    };
    treeData.value.push(newNode);
    newName.value = "新节点";

    nextTick(() => {
      const inputEl = inputRefs.value.get(newNode.id || newNode);
      if (inputEl) {
        inputEl.focus();
        inputEl.select();
      }
    });
  } else {
    const hasChildren = data.children;
    const nodeKey = data.id || data;
    const node = treeRef.value?.getNode(nodeKey);
    const isExpanded = node?.expanded; // 如果有子级且未展开，先展开节点
    if (hasChildren && !isExpanded) {
      if (
          treeRef.value &&
          treeRef.value.store &&
          treeRef.value.store.nodesMap[nodeKey]
      ) {
        treeRef.value.store.nodesMap[nodeKey].expanded = true;
      }
    }
    const newNode = {
      name: "新子节点",
      isEdit: true,
    };

    if (!data.children) {
      data.children = [];
    }
    data.children.push(newNode);
    newName.value = "新子节点";

    // 根据是否需要展开来决定延迟时间
    const delay = hasChildren && !isExpanded ? 200 : 50;

    // 等待DOM更新完成后再聚焦
    nextTick(() => {
      setTimeout(() => {
        const inputEl = inputRefs.value.get(newNode.id || newNode);
        if (inputEl) {
          inputEl.focus();
          inputEl.select();

          // 滚动到新增的节点位置
          inputEl.$el?.scrollIntoView?.({
            behavior: "smooth",
            block: "nearest",
          });
        }
      }, delay);
    });
  }
};

// 编辑归档
const handleEdit = (rows) => {
  try {
    row.value = { ...rows }; // 使用深拷贝避免引用问题
    dialogVisible.value = true;
    
    // 确保组件引用存在后再调用方法
    nextTick(() => {
      if (archiveDialogs.value && typeof archiveDialogs.value.editForm === 'function') {
        archiveDialogs.value.editForm(rows); // 调用编辑方法
      }
    });
  } catch (error) {
    console.error("编辑归档错误:", error);
    ElMessage.error("打开编辑界面失败");
  }
};

// 移除懒加载，直接获取数据
const getList = async () => {
  try {
    let res = await getTree();
    if (res.code === 200) {
      treeData.value = res.data?.records || res.data || [];
    } else {
      treeData.value = [];
    }
  } catch (error) {
    treeData.value = [];
  }
};
</script>
<style lang="scss" scoped>
.custom-tree-node {
  flex: 1;
  display: flex;
  align-items: center;
  justify-content: space-between;
  font-size: 14px;
  padding-right: 8px;
}

// 树形菜单样式
.tree-header {
  display: flex;
  justify-content: space-between;
  align-items: center;
  margin-bottom: 16px;
  padding-bottom: 12px;
  border-bottom: 1px solid #e4e7ed;

  h3 {
    margin: 0;
    font-size: 16px;
    font-weight: 600;
    color: #303133;
  }
}

.search-box {
  margin-bottom: 16px;

  .el-input {
    border-radius: 6px;

    :deep(.el-input__wrapper) {
      border-radius: 6px;
      box-shadow: 0 1px 3px rgba(0, 0, 0, 0.1);

      &:hover {
        box-shadow: 0 1px 6px rgba(0, 0, 0, 0.15);
      }

      &.is-focus {
        box-shadow: 0 0 0 2px rgba(64, 158, 255, 0.2);
      }
    }
  }
}

.tree-container {
  flex: 1;
  overflow-y: auto;
  border: 1px solid #dcdfe6;
  border-radius: 8px;
  background: #fff;
  box-shadow: 0 2px 8px rgba(0, 0, 0, 0.06);

  .custom-tree {
    padding: 8px;
    background: transparent;

    :deep(.el-tree-node) {
      .el-tree-node__content {
        height: 36px;
        padding: 0 8px;
        border-radius: 6px;
        margin: 2px 0;
        transition: all 0.2s ease;

        &:hover {
          background-color: #f0f9ff;
        }

        &.is-current {
          background-color: #e6f7ff;
          border: 1px solid #91d5ff;
        }
      }

      .el-tree-node__expand-icon {
        color: #606266;
        font-size: 14px;
        padding: 6px;

        &.expanded {
          transform: rotate(90deg);
        }

        &.is-leaf {
          color: transparent;
        }
      }
    }
  }
}

.tree-node-content {
  display: flex;
  align-items: center;
  width: 100%;
  padding: 4px 0;

  .node-icon {
    margin-right: 8px;
    color: #faad14;
    display: flex;
    align-items: center;

    .el-icon {
      font-size: 16px;

      &.expanded {
        color: #1890ff;
      }
    }
  }

  .node-label {
    flex: 1;
    min-width: 0;

    .label-text {
      font-size: 14px;
      color: #303133;
      cursor: pointer;
      display: block;
      overflow: hidden;
      text-overflow: ellipsis;
      white-space: nowrap;

      &:hover {
        color: #1890ff;
      }
    }
  }

  .node-actions {
    opacity: 0;
    transition: opacity 0.2s ease;
    display: flex;

    .el-button {
      padding: 4px;
      margin-left: 4px;
      color: #909399;
      min-height: auto;

      &:hover {
        color: #1890ff;
        background-color: #f0f9ff;
      }

      &.el-button--text:hover {
        background-color: #f0f9ff;
      }
    }
  }

  &:hover .node-actions {
    opacity: 1;
  }
}

// 输入框样式美化
.tree-input {
  flex: 1;

  :deep(.el-input__wrapper) {
    border-radius: 4px;
    border: 1px solid #d9d9d9;
    transition: all 0.2s ease;

    &:hover {
      border-color: #40a9ff;
    }

    &.is-focus {
      border-color: #1890ff;
      box-shadow: 0 0 0 2px rgba(24, 144, 255, 0.2);
    }
  }

  :deep(.el-input__inner) {
    padding: 4px 8px;
    font-size: 14px;
    color: #303133;

    &::placeholder {
      color: #bfbfbf;
    }
  }
}

.el-card {
  width: calc(100% - 40px);
  height: calc(100vh - 130px);
  margin: 20px;
  box-sizing: border-box;

  .left {
    width: 30%;
    height: calc(100vh - 160px);
    background-color: #fafafa;
    padding: 16px;
    float: left;
    box-sizing: border-box;
    border-radius: 8px;

    .left-content {
      width: 100%;
      height: 100%;
      display: flex;
      flex-direction: column;
    }
  }

  .right {
    width: 70%;
    height: calc(100vh - 160px);
    padding: 0 10px;
    float: left;
  }
}

.archive-management-card {
  margin: 0;
}
</style>
